/**
 * This file is a part of DesQ SNI.
 * DesQ SNI is a Widget to show SN Icons for DesQ Shell.
 * Copyright 2019-2022 Britanicus <marcusbritanicus@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#include "SNHost.hpp"

#include <wayqt/WayQtUtils.hpp>
#include <wayqt/Registry.hpp>
#include <wayqt/LayerShell.hpp>

#include <DFL/Application.hpp>

int main( int argc, char *argv[] ) {
    qputenv( "QT_WAYLAND_USE_BYPASSWINDOWMANAGERHINT", QByteArrayLiteral( "1" ) );
    DFL::Application *app = new DFL::Application( argc, argv );

    app->setOrganizationName( "DesQ" );
    app->setApplicationName( "SNI" );
    app->setApplicationVersion( "v" PROJECT_VERSION );

    WQt::Registry *registry = new WQt::Registry( WQt::Wayland::display() );
    registry->setup();

    if ( not app->lockApplication() ) {
        return 0;
    }

    StatusNotifierHost *sni = new StatusNotifierHost();

    sni->setStyleSheet( "#base {background: rgba(0, 0, 0, 0.5); border: 1px solid gray; border-radius: 5px; }" );
    sni->setLayoutStyle( StatusNotifierHost::Grid );
    sni->setMinimumWidth( 32 );
    sni->setMaximumWidth( 270 );
    sni->setMinimumHeight( 32 );
    sni->setButtonSize( 32 );
    sni->setMargins( QMargins( 3, 3, 3, 3 ) );

    Qt::WindowFlags wFlags = sni->windowFlags();

    sni->setWindowFlags( wFlags | Qt::BypassWindowManagerHint );
    sni->show();

    WQt::LayerShell::LayerType lyr  = WQt::LayerShell::Bottom;
    WQt::LayerSurface          *cls = registry->layerShell()->getLayerSurface( sni->windowHandle(), nullptr, lyr, "DesQSNI" );

    cls->setAnchors( WQt::LayerSurface::Top | WQt::LayerSurface::Left );

    /** Size of our surface */
    cls->setSurfaceSize( sni->size() );

    /** We'll keep a gap of 18 in on the left and top */
    cls->setMargins( QMargins( 18, 18, 0, 0 ) );

    /** Move this for panel or others */
    cls->setExclusiveZone( 0 );

    /** We may need keyboard interaction. */
    cls->setKeyboardInteractivity( WQt::LayerSurface::OnDemand );

    /** Commit to our choices */
    cls->apply();

    QObject::connect(
        sni, &StatusNotifierHost::resized, [ = ](){
            cls->setSurfaceSize( sni->size() );
            cls->apply();
        }
    );

    return app->exec();
}
